/* (C) 2016-2020 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {ServerState, INITIAL_SERVER_STATE} from "../state/ServerState";
import {ServerAction} from "../actions/ServerAction";
import {RSUtils} from "../RSUtils";

const icons = [
  "rs-fw rs-bigtab", // None
  "fa fa-bolt rs-fw rs-bigtab", // Bolt
  "fa fa-thermometer-half rs-fw rs-bigtab", // Temperature
  "fa fa-exclamation-triangle rs-fw rs-bigtab",
  "fa fa-battery-full rs-fw rs-bigtab", // Battery
  "fa fa-bug rs-fw rs-bigtab", // Bug
  "fa fa-plug rs-fw rs-bigtab", // Plug
  "fa fa-usb rs-fw rs-bigtab", // USB logo
  "rs rs-webcam rs-fw rs-bigtab", // Camera
  "fa fa-tasks rs-fw rs-bigtab",   // 9: Tasks
  "fa fa-bell rs-fw rs-bigtab", // Bell
  "fa fa-bluetooth rs-fw rs-bigtab", // 11: Bluetooth
  "fa fa-square-o rs-fw rs-bigtab", // 12: Square
  "fa fa-check-square-o rs-fw rs-bigtab", // 13: Checked Square
  "fa fa-hdd-o rs-fw rs-bigtab", // 14: HDD
  "fa fa-hourglass rs-fw rs-bigtab", // 15: Hourglass
  "fa fa-wifi rs-fw rs-bigtab", // 16: wifi
  "fa fa-microchip rs-fw rs-bigtab", // 17: Microchip
  'fa fa-clock-o rs-fw rs-bigtab' // 18: Clock
];

export function ServerReducer(state: ServerState = INITIAL_SERVER_STATE, action: any): ServerState {
  switch (action.type) {
    case ServerAction.SET_LANGUAGES:
      return Object.assign({}, state, {languages: action.payload});
    case ServerAction.SET_EXTERNAL_COMMANDS:
      return Object.assign({}, state, {externalCommands: action.payload});
    case ServerAction.SET_WEB_CALLS:
      return Object.assign({}, state, {webCalls: action.payload});
    case ServerAction.SET_ALL_EXTERNAL_COMMANDS:
      return Object.assign({}, state, {allExternalCommands: action.payload});
    case ServerAction.SET_GLOBAL: {
      const data: any = {};
      data[action.payload.key] = action.payload.data;
      return Object.assign({}, state, Object.assign({}, state.globals, data));
    }
    case ServerAction.SET_SETTINGS:
      return Object.assign({}, state, {settings: Object.assign({}, state.settings, action.payload), settingsLoaded: true});
    case ServerAction.INIT_SETTING: {
      if (state.settings[action.payload.key] !== undefined) {
        return;
      }
      const data: any = {};
      data[action.payload.key] = action.payload.data;
      return Object.assign({}, state, Object.assign({}, state.settings, data));
    }
    case ServerAction.UPDATE_SETTINGS: {
      if (state.settings[action.payload.key] !== undefined) {
        return;
      }
      const data: any = {};
      data[action.payload.key] = action.payload.data;
      return Object.assign({}, state, Object.assign({}, state.settings, data));
    }
    case ServerAction.SET_SERVER_FOLDERS:
      return Object.assign({}, state, {serverFolders: RSUtils.copyArray(action.payload)});
    case ServerAction.SET_SCREEN_LOCK:
      return Object.assign({}, state, {screenLocked: action.payload});
    case ServerAction.SET_DISK_USAGE: {
      let cName = "";
      const usage = 100 - 100 * (action.payload.available / action.payload.capacity);
      if (action.payload.capacity < 1000 * 1024 * 1024) {
        cName = "progress-bar-warning";
      } else if (action.payload.capacity < 500 * 1024 * 1024) {
        cName = "progress-bar-danger";
      } else {
        cName = "progress-bar-success";
      }
      return Object.assign({}, state, {diskUsage: Object.assign({usage: usage, classname: cName}, action.payload)});
    }
    case ServerAction.SET_HARDWARE_INFO: {
      let hIcon = "";
      switch (action.payload.maxUrgency) {
        case 0:
        case 1:
          hIcon = "fa fa-bolt";
          break;
        case 2:
          hIcon = "fa fa-bolt text-warning-bg";
          break;
        case 3:
          hIcon = "fa fa-bolt text-error-bg";
          break;
      }
      action.payload.list.forEach((hi) => {
        const extra = hi.url ? "" /* " &nbsp; <i class=\"fa fa-external-link\"></i>" */ : "";
        // 0 = text, 1 = int value, 2 = 1 digit info, 3 = 2 digit info, 10 = splitter, 11 = Block Headline
        switch (hi.msgType) {
          case 0:
            hi.content = hi.name + " " + hi.text + extra;
            break;
          case 1:
            hi.content = hi.name + " " + parseInt(hi.value, 10) + " " + hi.unit + extra;
            break;
          case 2:
            hi.content = hi.name + " " + parseFloat(hi.value).toFixed(1) + " " + hi.unit + extra;
            break;
          case 3:
            hi.content = hi.name + " " + parseFloat(hi.value).toFixed(1) + " " + hi.unit + extra;
            break;
        }
        hi.iconClass = icons[hi.icon];
        if (hi.urgency === 2) {
          hi.iconClass += " text-warning-bg";
        }
        if (hi.urgency === 3) {
          hi.iconClass += " text-error-bg";
        }
      });
      return Object.assign({}, state, {
          hardwareInfoMaxUrgency: action.payload.maxUrgency,
          hardwareInfoIcon: hIcon,
          hardwareInfos: action.payload.list
      });
    }
    default:
      return state;
  }
}
