/* (C) 2016-2020 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Component, ViewChild, ElementRef, OnDestroy} from "@angular/core";
import {PrinterState, PrintersState, PrinterAction} from "binding";
import {RSGCodeKeyboard, RSSelect} from "front-ui";
import {select} from "@angular-redux/store";
import {Observable, SubscriptionLike} from "rxjs";

@Component({
  selector: "rs-printer-commands",
  template: `
<rs-page>
  <rs-side>
    <rs-sideicon title="<?php _('Home') ?>" icon="fa fa-home" link="/"></rs-sideicon>
  </rs-side>
  <rs-side>
      <rs-sideicon *rsIfPrinting="true" title="<?php _('Print') ?>" icon="rs rs-print" link="/print-status"></rs-sideicon>
      <rs-sideicon *rsIfPrinting="false" icon="rs rs-move-xy" title="<?php _('Move') ?>" link="/move-tool"></rs-sideicon>
  </rs-side>
  <rs-side>
      <rs-sideicon *rsIfConnected="true" icon="rs rs-params" title="<?php _('Status') ?>" link="/params"></rs-sideicon>
  </rs-side>
<main>
<rs-headline><i class="rs rs-printer-commands"></i> <?php _('Console') ?></rs-headline>
<div *rsIfConnected="true" class="content-h1 log-container">
    <div class="regular-grid margin-leftright">
        <div class="col-2 margin-bottom">
            <rs-button class="secondary full" style="margin:auto;white-space: nowrap;overflow: hidden" (hit)="selectGCode()"><?php _('Send G-Code') ?></rs-button>
        </div>
        <div class="col-2 margin-bottom" *ngIf="printer?.config?.quickCommands?.length>0">
            <rs-button class="secondary full" style="margin:auto;white-space: nowrap;overflow: hidden" (hit)="selectCommand()"><?php _('Send Predefined') ?><span class="caret"></span> </rs-button>
        </div>
    </div>
    <div class="regular-grid margin-leftright">
        <div class="col-3 margin-bottom" style="margin:auto;white-space: nowrap;overflow: hidden"><rs-switch (checkedChange)="toggleCommands($event)" [checked]="printer.log.logCommands" style="margin:auto;white-space: nowrap;overflow: hidden"></rs-switch> <?php _('CMDs') ?></div>
        <div class="col-3 margin-bottom" style="margin:auto;white-space: nowrap;overflow: hidden"><rs-switch (checkedChange)="toggleACK($event)" [checked]="printer.log.logACK" style="margin:auto;white-space: nowrap;overflow: hidden"></rs-switch> <?php _('ACK') ?></div>
        <div class="col-3 margin-bottom" style="margin:auto;white-space: nowrap;overflow: hidden"><rs-switch (checkedChange)="togglePause($event)" [checked]="printer.log.paused" style="margin:auto;twhite-space: nowrap;overflow: hidden"></rs-switch> <?php _('Pause') ?></div>
    </div>
    <div class="log" #log rsDrag>
    </div>
</div>
</main>
</rs-page>`
})
export class ConsoleComponent implements OnDestroy {
  printer: PrinterState;
  active: PrinterState = undefined;
  @select("printers") printers$: Observable<PrintersState>;
  @ViewChild("log") logElement: ElementRef;
  lastLogId = -1;
  logUpdatePlanned: any = false;

  private printersSubscription: SubscriptionLike;

  constructor(private printerAction: PrinterAction,
              private stringKbd: RSGCodeKeyboard,
              public rsselect: RSSelect) {
    this.printersSubscription = this.printers$.subscribe((p: PrintersState) => {
      if (p.active) {
        this.printer = p.printers[p.active];
        if (this.logUpdatePlanned === false && this.printer.log.lastLogStart !== this.lastLogId && this.logElement) {
          this.logUpdatePlanned = setTimeout(() => { this.updateLogView(); }, 500);
          this.lastLogId = this.printer.log.lastLogStart;
          const elem = this.logElement.nativeElement;
          elem.innerHTML = this.printer.log.history.join("");
          if (elem) {
            elem.scrollTop = elem.scrollHeight;
          }
        }
      } else {
        this.printer = undefined;
      }
    });
  }

  updateLogView(): void {
    this.lastLogId = this.printer.log.lastLogStart;
    const elem = this.logElement.nativeElement;
    elem.innerHTML = this.printer.log.history.join("");
    if (elem) {
      elem.scrollTop = elem.scrollHeight;
    }
    this.logUpdatePlanned = false;
  }

  ngOnDestroy(): void {
    this.printersSubscription.unsubscribe();
    if (this.logUpdatePlanned !== false) {
      clearTimeout(this.logUpdatePlanned);
    }
  }

  // send gcode directly
  selectGCode(): void {
    this.stringKbd.getString("", "<?php _('G-Code to Send') ?>", {}).then((cmd) => {
      this.printerAction.sendGCode(this.printer.slug, cmd);
    }, () => {
    });
  }

  // select predefined command
  selectCommand(): void {
    this.rsselect.select(this.printer.config.quickCommands, "name", "<?php _('Select Command') ?>").then((data) => {
      this.printerAction.sendQuickCommand(this.printer.slug, data.name);
    });
  }

  toggleCommands(v: boolean): void {
    this.printerAction.setLogCommands(this.printer.slug, v);
  }

  toggleACK(v: boolean): void {
    this.printerAction.setLogACK(this.printer.slug, v);
  }

  togglePause(v: boolean): void {
    this.printerAction.setLogPause(this.printer.slug, v);
  }
}
